use crate::handlers;
use crate::state::AppState;
use axum::{routing, Router};
use serde::{Deserialize, Serialize};
use std::{
    net::SocketAddr,
    sync::{Arc, Mutex},
};
use tokio::sync::mpsc;
use tokio::task::JoinHandle;

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct ServerSettings {
    pub port: String,
}

#[derive(Debug, Clone)]
pub enum ServerMessage {
    UpdateSettings(ServerSettings),
    Interact,
    Shutdown,
    // Add other message types here
}

pub async fn run_server(app_state: AppState, mut rx: mpsc::Receiver<ServerMessage>) {
    let mut server_handle = start_server(&app_state);

    loop {
        tokio::select! {
            _ = &mut server_handle => {
                // Server has stopped
                eprintln!("Server stopped.");
                break;
            },
            Some(message) = rx.recv() => {
                match message {
                    // This is the message type that actually restarts the server.
                    ServerMessage::UpdateSettings(new_settings) => {
                        println!("Received new settings: {:?}", new_settings);
                        let mut settings_guard = app_state.settings.lock().unwrap();
                        *settings_guard = new_settings;

                        // Restart the server with new settings
                        println!("Aborting current server...");
                        server_handle.abort(); // Cancel the previous server task
                        println!("Starting new server...");
                        drop(settings_guard); // Ensure the lock is released before starting the new server
                        server_handle = start_server(&app_state);
                        println!("Server started.");
                    },
                    ServerMessage::Shutdown => {
                        // Handle server shutdown
                        println!("Shutting down server...");
                        let mut should_run_guard = app_state.should_run.lock().unwrap();
                        *should_run_guard = false;
                        server_handle.abort();
                        println!("Server shutdown.");
                        break;
                    },
                    // Handle other message types here
                    ServerMessage::Interact => {
                        // Example interaction: print current settings
                        let settings_guard = app_state.settings.lock().unwrap();
                        println!("Current settings: {:?}", *settings_guard);
                    },
                }
            }
        }
    }
}

fn start_server(app_state: &AppState) -> JoinHandle<()> {
    // Compute the address before entering the async block
    let addr = {
        let settings = app_state.settings.lock().unwrap();
        let port: u16 = settings.port.parse().unwrap_or(3311);
        SocketAddr::from(([0, 0, 0, 0], port))
    };

    let server_app_state = app_state.clone();

    println!("listening on {}", addr);

    tokio::spawn(async move {
        let app = Router::new()
            .route("/", routing::get(handlers::home))
            .route("/play/:hash", routing::get(handlers::play_handler))
            .route("/stop", routing::get(handlers::stop_handler))
            .with_state(server_app_state.clone());

        let server = axum::Server::bind(&addr).serve(app.into_make_service());

        // Run the server and monitor the should_run flag
        tokio::select! {
            _ = server => {
                eprintln!("Server...");
            },
            _ = monitor_shutdown(server_app_state.should_run.clone()) => {
                println!("Server shutdown signal received");
            }
        }
    })
}

async fn monitor_shutdown(should_run: Arc<Mutex<bool>>) {
    eprintln!("Monitoring shutdown...");
    while *should_run.lock().unwrap() {
        tokio::time::sleep(tokio::time::Duration::from_secs(1)).await;
    }
    eprintln!("Shutdown completed.");
}
